using System.Text.Json;
using System.Text.Json.Serialization;

namespace File_Watcher.Models.Binder;

public class EAFLogConfiguration
{

    public string? SearchPattern { get; set; }

    public override string ToString()
    {
        string result = JsonSerializer.Serialize(this, BinderEAFLogConfigurationSourceGenerationContext.Default.EAFLogConfiguration);
        return result;
    }

    private static void PreVerify(IConfigurationRoot configurationRoot, EAFLogConfiguration? configuration)
    {
        if (configuration?.SearchPattern is null)
        {
            List<string> paths = [];
            foreach (IConfigurationProvider configurationProvider in configurationRoot.Providers)
            {
                if (configurationProvider is not Microsoft.Extensions.Configuration.Json.JsonConfigurationProvider jsonConfigurationProvider)
                    continue;
                if (jsonConfigurationProvider.Source.FileProvider is not Microsoft.Extensions.FileProviders.PhysicalFileProvider physicalFileProvider)
                    continue;
                paths.Add(physicalFileProvider.Root);
            }
            throw new NotSupportedException($"Not found!{Environment.NewLine}{string.Join(Environment.NewLine, paths.Distinct())}");
        }
    }

    private static void Verify(EAFLogConfiguration _)
    {
    }

    private static Models.EAFLogConfiguration Get(EAFLogConfiguration? configuration)
    {
        Models.EAFLogConfiguration result;
        if (configuration is null) throw new NullReferenceException(nameof(configuration));
        if (configuration.SearchPattern is null) throw new NullReferenceException(nameof(configuration.SearchPattern));
        Verify(configuration);
        result = new(configuration.SearchPattern);
        return result;
    }

    public static Models.EAFLogConfiguration Get(IConfigurationRoot configurationRoot)
    {
        Models.EAFLogConfiguration result;
        IConfigurationSection configurationSection = configurationRoot.GetSection(nameof(Models.EAFLogConfiguration));
#pragma warning disable IL3050, IL2026
        EAFLogConfiguration? configuration = configurationSection.Get<EAFLogConfiguration>();
#pragma warning restore IL3050, IL2026
        PreVerify(configurationRoot, configuration);
        result = Get(configuration);
        return result;
    }

}

[JsonSourceGenerationOptions(WriteIndented = true)]
[JsonSerializable(typeof(EAFLogConfiguration))]
internal partial class BinderEAFLogConfigurationSourceGenerationContext : JsonSerializerContext
{
}